==> MRAConnector-0.82.tar.bz2: MRAConnector-0.82/core/Smdn.Net.Pop3.Client/Smdn.Net.Pop3.Client/PopClient.OperationsOneShot.cs <==
//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2010-2012 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Collections.Generic;
using System.IO;
#if NET_3_5
using System.Linq;
#else
using Smdn.Collections;
#endif
using System.Text;

namespace Smdn.Net.Pop3.Client {
  partial class PopClient {
    public static long GetMessageCount(Uri popUrl,
                                       string password)
    {
      return Download(popUrl, password, false,
                      delegate(PopClient client) {
        return client.MessageCount;
      });
    }

    public static string DownloadFirstMessageAsText(Uri popUrl,
                                                    string password,
                                                    bool deleteAfterDownload,
                                                    Encoding encoding)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAsText(encoding);
      });
    }

    public static string DownloadLastMessageAsText(Uri popUrl,
                                                   string password,
                                                   bool deleteAfterDownload,
                                                   Encoding encoding)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAsText(encoding);
      });
    }

    public static string[] DownloadAllMessagesAsText(Uri popUrl,
                                                     string password,
                                                     bool deleteAfterDownload,
                                                     Encoding encoding)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesAsText(encoding).ToArray();
      });
    }

    public static byte[] DownloadFirstMessageAsByteArray(Uri popUrl,
                                                         string password,
                                                         bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAsByteArray();
      });
    }

    public static byte[] DownloadLastMessageAsByteArray(Uri popUrl,
                                                        string password,
                                                        bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAsByteArray();
      });
    }

    public static Stream DownloadFirstMessageAsStream(Uri popUrl,
                                                      string password,
                                                      bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAsStream();
      });
    }

    public static Stream DownloadLastMessageAsStream(Uri popUrl,
                                                     string password,
                                                     bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAsStream();
      });
    }

    public static TOutput DownloadFirstMessageAs<TOutput>(Uri popUrl,
                                                          string password,
                                                          bool deleteAfterDownload,
                                                          Converter<Stream, TOutput> converter)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAs(converter);
      });
    }

    public static TResult DownloadFirstMessageAs<T, TResult>(Uri popUrl,
                                                             string password,
                                                             bool deleteAfterDownload,
                                                             Func<Stream, T, TResult> read, T arg)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAs(read, arg);
      });
    }

    public static TOutput DownloadLastMessageAs<TOutput>(Uri popUrl,
                                                         string password,
                                                         bool deleteAfterDownload,
                                                         Converter<Stream, TOutput> converter)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAs(converter);
      });
    }

    public static TResult DownloadLastMessageAs<T, TResult>(Uri popUrl,
                                                            string password,
                                                            bool deleteAfterDownload,
                                                            Func<Stream, T, TResult> read, T arg)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAs(read, arg);
      });
    }

    public static TOutput[] DownloadAllMessagesAs<TOutput>(Uri popUrl,
                                                           string password,
                                                           bool deleteAfterDownload,
                                                           Converter<Stream, TOutput> converter)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesAs(converter).ToArray();
      });
    }
    
    public static TResult[] DownloadAllMessagesAs<T, TResult>(Uri popUrl,
                                                              string password,
                                                              bool deleteAfterDownload,
                                                              Func<Stream, T, TResult> read, T arg)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesAs(read, arg).ToArray();
      });
    }

    public static void DownloadFirstMessageToFile(Uri popUrl,
                                                  string password,
                                                  bool deleteAfterDownload,
                                                  string path)
    {
      Download(popUrl, password, deleteAfterDownload,
               delegate(PopClient client) {
        client.DownloadFirstMessageToFile(path);

        return true; // dummy
      });
    }

    public static void DownloadLastMessageToFile(Uri popUrl,
                                                 string password,
                                                 bool deleteAfterDownload,
                                                 string path)
    {
      Download(popUrl, password, deleteAfterDownload,
               delegate(PopClient client) {
        client.DownloadLastMessageToFile(path);

        return true; // dummy
      });
    }

    public static void DownloadAllMessagesToDirectory(Uri popUrl,
                                                      string password,
                                                      bool deleteAfterDownload,
                                                      string directory)
    {
      Download(popUrl, password, deleteAfterDownload,
               delegate(PopClient client) {
        client.DownloadAllMessagesToDirectory(directory);

        return true; // dummy
      });
    }

    public static void DownloadAllMessagesToFiles(Uri popUrl,
                                                  string password,
                                                  bool deleteAfterDownload,
                                                  Func<PopMessageInfo, string> selectFileName)
    {
      Download(popUrl, password, deleteAfterDownload,
               delegate(PopClient client) {
        client.DownloadAllMessagesToFiles(selectFileName);

        return true; // dummy
      });
    }

    private static TResult Download<TResult>(Uri popUrl,
                                             string password,
                                             bool deleteAfterDownload,
                                             Func<PopClient, TResult> getResult)
    {
      if (popUrl == null)
        throw new ArgumentNullException("popUrl");

      using (var client = new PopClient(popUrl)) {
        client.Connect(password);

        client.DeleteAfterRetrieve = deleteAfterDownload;

        var ret = getResult(client);

        client.Logout();

        return ret;
      }
    }
  }
}

==> MRAConnector-0.84.tar.bz2: MRAConnector-0.84/core/Smdn.Net.Pop3.Client/Smdn.Net.Pop3.Client/PopClient.OperationsOneShot.cs <==
//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2010-2012 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Collections.Generic;
using System.IO;
#if NET_3_5
using System.Linq;
#else
using Smdn.Collections;
#endif
using System.Text;

namespace Smdn.Net.Pop3.Client {
  partial class PopClient {
    public static long GetMessageCount(Uri popUrl,
                                       string password)
    {
      return Download(popUrl, password, false,
                      delegate(PopClient client) {
        return client.MessageCount;
      });
    }

    public static string DownloadFirstMessageAsText(Uri popUrl,
                                                    string password,
                                                    bool deleteAfterDownload,
                                                    Encoding encoding)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAsText(encoding);
      });
    }

    public static string DownloadLastMessageAsText(Uri popUrl,
                                                   string password,
                                                   bool deleteAfterDownload,
                                                   Encoding encoding)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAsText(encoding);
      });
    }

    public static string[] DownloadAllMessagesAsText(Uri popUrl,
                                                     string password,
                                                     bool deleteAfterDownload,
                                                     Encoding encoding)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesAsText(encoding).ToArray();
      });
    }

    public static byte[] DownloadFirstMessageAsByteArray(Uri popUrl,
                                                         string password,
                                                         bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAsByteArray();
      });
    }

    public static byte[] DownloadLastMessageAsByteArray(Uri popUrl,
                                                        string password,
                                                        bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAsByteArray();
      });
    }

    public static Stream DownloadFirstMessageAsStream(Uri popUrl,
                                                      string password,
                                                      bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAsStream();
      });
    }

    public static Stream DownloadLastMessageAsStream(Uri popUrl,
                                                     string password,
                                                     bool deleteAfterDownload)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAsStream();
      });
    }

    public static TOutput DownloadFirstMessageAs<TOutput>(Uri popUrl,
                                                          string password,
                                                          bool deleteAfterDownload,
                                                          Converter<Stream, TOutput> converter)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAs(converter);
      });
    }

    public static TResult DownloadFirstMessageAs<T, TResult>(Uri popUrl,
                                                             string password,
                                                             bool deleteAfterDownload,
                                                             Func<Stream, T, TResult> read, T arg)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageAs(read, arg);
      });
    }

    public static TOutput DownloadLastMessageAs<TOutput>(Uri popUrl,
                                                         string password,
                                                         bool deleteAfterDownload,
                                                         Converter<Stream, TOutput> converter)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAs(converter);
      });
    }

    public static TResult DownloadLastMessageAs<T, TResult>(Uri popUrl,
                                                            string password,
                                                            bool deleteAfterDownload,
                                                            Func<Stream, T, TResult> read, T arg)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageAs(read, arg);
      });
    }

    public static TOutput[] DownloadAllMessagesAs<TOutput>(Uri popUrl,
                                                           string password,
                                                           bool deleteAfterDownload,
                                                           Converter<Stream, TOutput> converter)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesAs(converter).ToArray();
      });
    }
    
    public static TResult[] DownloadAllMessagesAs<T, TResult>(Uri popUrl,
                                                              string password,
                                                              bool deleteAfterDownload,
                                                              Func<Stream, T, TResult> read, T arg)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesAs(read, arg).ToArray();
      });
    }

    public static FileInfo DownloadFirstMessageToFile(Uri popUrl,
                                                      string password,
                                                      bool deleteAfterDownload,
                                                      string path)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadFirstMessageToFile(path);
      });
    }

    public static FileInfo DownloadLastMessageToFile(Uri popUrl,
                                                     string password,
                                                     bool deleteAfterDownload,
                                                     string path)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadLastMessageToFile(path);
      });
    }

    public static FileInfo[] DownloadAllMessagesToDirectory(Uri popUrl,
                                                            string password,
                                                            bool deleteAfterDownload,
                                                            string directory)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesToDirectory(directory);
      });
    }

    public static FileInfo[] DownloadAllMessagesToFiles(Uri popUrl,
                                                        string password,
                                                        bool deleteAfterDownload,
                                                        Func<PopMessageInfo, string> selectFileName)
    {
      return Download(popUrl, password, deleteAfterDownload,
                      delegate(PopClient client) {
        return client.DownloadAllMessagesToFiles(selectFileName);
      });
    }

    private static TResult Download<TResult>(Uri popUrl,
                                             string password,
                                             bool deleteAfterDownload,
                                             Func<PopClient, TResult> getResult)
    {
      if (popUrl == null)
        throw new ArgumentNullException("popUrl");

      using (var client = new PopClient(popUrl)) {
        client.Connect(password);

        client.DeleteAfterRetrieve = deleteAfterDownload;

        var ret = getResult(client);

        client.Logout();

        return ret;
      }
    }
  }
}
